#ifndef MPLIB1_BPO_INTERNAL
#define MPLIB1_BPO_INTERNAL
/*
*******************************************************************************
*  Copyright (c) 1996 Martin Poole                            *
*******************************************************************************
**
*******************************************************************************
*/

#ifndef MPLIB1_CONFIG
#include <mplib1/mplib1_config.h>
#endif

#ifndef MPLIB1_DL_LIST
#include <mplib1/dl_list.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>

#include <mplib1/bpo.h>

/* ------------------------------------------------------------------
    defines
   ------------------------------------------------------------------ */

#define GNNODE(n,off)	((bpo_Node_t *)GRPTR(n,off))

#define GANNODE(bp,off)	(bpo_Node_t *)((void *)bp+off)



#define SODB_MAGIC		0x536d6567

#define SODB_VERSION_1		1
#define SODB_VERSION_2		2
#define SODB_VERSION_4		4
#define SODB_CURRENT_VERSION	SODB_VERSION_4

#define SODB_NAME_SIZE1		512

enum sodb_types
{
SODB_T_UNKNOWN,
SODB_T_MMAP,
SODB_T_IPC
};


#define MIN_SHALLOC_SIZE	32
#define MIN_SHALLOC_MASK	0xffffffe0

#define STD_NUM_TRACK_BLKS	100

#ifndef BIT
#define BIT(n)			(1<<(n))
#endif

#define SHM_INHERIT_GROUP	BIT(0)
#define SHM_INHERIT_PROGRAM	BIT(1)
#define SHM_AUTOMAGIC_PIPE	BIT(2)

#define SHM_CHECK_ALL_PROCESSES		1
#define SHM_CHECK_THIS_PROCESS		2
#define SHM_CHECK_DEAD_PROCESSES	3

/* Return values to the validate processes inform call */
#define SHM_LEAVE_PROC	0
#define SHM_DELETE_PROC	1

#ifndef PID_STR_LEN
#define PID_STR_LEN	12
#endif

/* ------------------------------------------------------------------
    structures
   ------------------------------------------------------------------ */

struct sodb_resource
{
bpo_Node_t	res_node;
off_t		res_off;
};

struct sodb
{
int	sodb_magic;
char	sodb_use_name[SODB_NAME_SIZE1];
size_t	sodb_size;
enum	sodb_types	sodb_type;
int	sodb_version;
int	sodb_flags;

struct	stat		mmap_stat;
int	mmap_fd;
int	mmap_prot;
int	mmap_flags;
off_t	mmap_off;
char	mmap_name[SODB_NAME_SIZE1];

key_t	ipc_key;
char	ipc_path[SODB_NAME_SIZE1];
char	ipc_tok_id;
int	ipc_id;

struct	bpo_pid_lock	res_p_lock;
bpo_List_t		sodb_resource_list;

struct	sodb_resource	sodb_resource;
char	sodb_res_name[10];	/* this has "SODB Base" copied into it */

off_t	shalloc_offset;
size_t	shalloc_size;

};

struct sodb_track
{
dl_Node_t	st_Node;
struct	sodb	*sodbp;
int		mmap_fd;
const char	*upper;
mode_t		access_mode;
char		fname[1];
};

struct sodb_res_find
{
struct	sodb	*sodbp;
const	char	*name;
void		*ptr;
void		*obj;
int		flags;
int		ierrno;
off_t		rv;
};


/* ------------------------------------------------------------------
    function definitions
   ------------------------------------------------------------------ */

extern void Init_SODB_Resources( struct sodb *sodbp );


/* ------------------------------------------------------------------
    structures
   ------------------------------------------------------------------ */

struct shalloc_track_head
{
off_t	lno_me;
off_t	next_track_head;
size_t	head_and_tracks;
size_t	num_tracks;
size_t	num_empty;
int	sth_flags;
};

struct shalloc_track
{
off_t	sh_off;
size_t	sh_size;
};

struct shalloc_base
{
off_t	lno_me;
size_t	sh_size;
off_t	lno_first_free;
off_t	lno_first_used;
b_lock_t	sb_lock;
struct	shalloc_track	emergency_used;
struct	sodb_resource	shalloc_resource;
char	shalloc_name[10];
};

/* ------------------------------------------------------------------
    function definitions
   ------------------------------------------------------------------ */

extern struct sodb_resource *
shalloc_init( void *seg_base, off_t shalloc_base_off, size_t shalloc_sz );



/* ------------------------------------------------------------------
    structures
   ------------------------------------------------------------------ */

struct Q_Head
{
off_t			lno_me;
bpo_List_t		Qh_List;
};

struct bpo_Q_pid
{
bpo_Node_t	Q_pid_Node;
pid_t		Q_pid;
char		Q_pidstr[PID_STR_LEN];
};

struct bpo_private_pid
{
dl_Node_t		q_Node;
pid_t		pid;
char		pid_str[PID_STR_LEN];
};

struct bpo_private_q
{
dl_Node_t		q_Node;
int		Q_max_count;
int		Q_Count;
int		Q_notify;
int		Q_flags;
pid_t		Q_sole_pid;
dl_List_t		private_pids;
unsigned long	pid_ser_nbr;
dl_List_t		old_pids;
struct		bpo_Q	*qp;
char		Q_name[1];
};

struct cache_queue_list
{
dl_Node_t		lnk_node;
char		ptr_str[40];
unsigned long	serial_nbr;
time_t		update_tm;
struct	Q_Head	*qh;
dl_List_t		q_list;
dl_List_t		old_q_list;
};


typedef int (*Validate_Queues_t)(struct bpo_private_q *pqp, void *param1 );


/* ------------------------------------------------------------------
    function definitions
   ------------------------------------------------------------------ */

extern struct Q_Head *
get_Q_resource( const void *hint );

extern struct cache_queue_list *
cache_queues( struct Q_Head *qh, const char *q_name );

extern void
free_q_stuff( struct bpo_Q *qp );

extern void
free_this_private_q( struct bpo_private_q *pqp );

extern int
Validate_Queues( const void *hint,
		int p_which_queues,
		Validate_Queues_t disp,
		void *param1 );



/* ------------------------------------------------------------------
    structures
   ------------------------------------------------------------------ */

struct shm_proc_list
{
off_t           lno_me;
int		serial_nbr;
time_t		update_tm;
bpo_List_t	proc_List;
};

struct shm_process
{
off_t		lno_me;
bpo_Node_t	proc_Node;
pid_t		pid;
time_t		pid_start_tm;
bpo_List_t	pid_resources;
char		pid_str[PID_STR_LEN];
off_t		pid_grp_o;
off_t		pid_nm_o;
off_t		pid_fifo_o;
};

struct shm_process_private
{
dl_Node_t	spp_Node;
pid_t	pid;
char	pid_str[PID_STR_LEN];
char	*pid_grp;
char	*pid_nm;
char	*pid_fifo;
int	fifo_fd;
int	fifo_created;
mode_t	fifo_mode;
time_t	pid_start_tm;
struct	shm_process	*spp;
};

struct cache_proc_list
{
dl_Node_t	lnk_node;
char	ptr_str[40];
unsigned	long	serial_nbr;
time_t	update_tm;
struct	shm_proc_list	*splp;
dl_List_t	proc_list;
dl_List_t	old_proc_list;
};

typedef int (* Validate_Processes_t)( struct shm_process_private *shmpp,
					void *param1 );

/* ------------------------------------------------------------------
    function definitions
   ------------------------------------------------------------------ */

/* Allocate functions */
extern struct shm_process_private *
alloc_private_process( pid_t the_pid, const char *pid_grp,
			const char *pid_nm, const char *pid_fifo );
extern void
free_private_process( struct shm_process_private *sppp );

#ifndef MPLIB1_BPO_PROC
extern struct shm_process_private * Get_My_Proc_Details( mode_t privs );
extern struct shm_process_private * Check_My_Proc_Details( void );
#endif

extern int
Validate_Processes( const void * hint, int which_procs,
		    Validate_Processes_t disp,
		    void *param1 );

extern struct shm_proc_list *Find_Proc_List( const void *hint );
extern struct cache_proc_list *Get_Cache_Procs( const void *hint );

#endif
/* -- End of File -- */
