/*
*******************************************************************************
*  Copyright (c) 1996 Martin Poole                            *
*******************************************************************************
**
*******************************************************************************
*/

/* ------------------------------------------------------------------
    Include files
   ------------------------------------------------------------------ */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <strings.h>

#include <mplib1/mplib1_config.h>
#include <mplib1/safe_string.h>
#include <mplib1/fgetline.h>
#include <mplib1/dl_list.h>
#include <mplib1/cfg_file.h>
#include <mplib1/timestamp.h>
#include <include/cfg_file_internal.h>

/* ------------------------------------------------------------------
    defines
   ------------------------------------------------------------------ */

static char config_list[]="config_items";
static int done_init=0;
static struct config_list *def_list=NULL;
static dl_List_t current_lists;

/* ------------------------------------------------------------------
    Code starts here
   ------------------------------------------------------------------ */

struct config_list *
get_config_list( const char *list_name, int create )
{
    struct config_list *clp;

    if (!done_init)
    {
	done_init = 1;
	(void)dl_Init_List( &current_lists, 0 );
    }
    if (list_name==NULL)
    {
	if (def_list)
	    return(def_list);
	list_name = config_list;
    }

    clp = dl_Find_Item_By_Name( &current_lists, list_name );

    if (clp==NULL && create)
    {
	clp = malloc( sizeof(struct config_list) + Sstrlen(list_name) );
	if (clp)
	{
	    if (list_name==config_list)
		def_list = clp;
	    Sstrcpy( clp->cl_name, list_name );
	    clp->cl_flags=0;
	    (void)dl_Init_Node( &clp->cl_Node, clp->cl_name, clp );
	    (void)dl_Init_List( &clp->cl_Items, 0 );
	    (void)dl_Add_Head( &current_lists, &clp->cl_Node );
	}
    }
    return(clp);
}

dl_List_t *
get_config_list_list( void )
{
    if (!done_init)
    {
	done_init = 1;
	(void)dl_Init_List( &current_lists, 0 );
    }
    return( &current_lists );
}


static struct config_item *
Alloc_Item( const char *key, const char *val )
{
    struct config_item *cip;
    size_t slen;
    char *cp;

    slen = Sstrlen(val) + 1;
    cip = malloc( sizeof(struct config_item) + Sstrlen(key) + slen );
    if (cip)
    {
	cp = cip->ci_val + slen;
	Sstrcpy( cip->ci_val, val );
	Sstrcpy( cp, key );
	cip->eval_str=NULL;
	(void)dl_Init_Node( &cip->ci_Node, cp, cip );
    }
    return(cip);
}

int
set_private_string( const char *list_name, const char *key, const char *val )
{
    struct config_list *clp;
    struct config_item *cip;
    int rv=0;

    clp = get_config_list( list_name, 1 );
    if (clp)
    {
	cip = (struct config_item *)dl_Find_Item_By_Name( &clp->cl_Items, key );
	if (cip==NULL || Sstrlen(cip->ci_val) < Sstrlen(val))
	{
	    if (cip)
	    {
		(void)dl_Remove_Node( &cip->ci_Node );
		free( cip );
	    }
	    cip = Alloc_Item( key, val );
	    if (cip)
	    {
		(void)dl_Add_Head( &clp->cl_Items, &cip->ci_Node );
		rv=1;
	    }
	}else
	{
	    /* replace the existing text */
	    Sstrcpy( cip->ci_val, val );
	    rv=1;
	}
    }
    return(rv);
}

int
read_private_string( const char *list_name, const char *line )
{
    int rv=0;
    char *newline,*cp=NULL,*szs=NULL;
    static char cl_seps[]=",: \t=";

    newline = strdup(line);
    /* first we parse the line for various parameters */
    if ( (cp=strtok(newline,cl_seps)) )
	szs=strtok(NULL,"\015");

    if (cp && *cp!='#' )
    {
	rv = set_private_string( list_name, cp, szs );
    }
    free(newline);

    return(rv);
}


void
flush_item_eval( const char *list_name, const char *item )
{
    struct config_list *clp;
    struct config_item *cip;
    if (   (clp = get_config_list( list_name, 0 ))
	&& (cip = dl_Find_Item_By_Name( &clp->cl_Items, item ))
	&& (cip->eval_str != NULL)
       )
    {
	free(cip->eval_str);
	cip->eval_str=NULL;
    }
    return;
}


static void
flush_this_eval( struct config_item *cip )
{
    if (cip->eval_str)
    {
	free(cip->eval_str);
	cip->eval_str=NULL;
    }
    return;
}

int
flush_list_evals( const char *list_name )
{
    struct config_list *clp;
    int rv=0;
    clp = get_config_list( list_name, 0 );
    if (clp)
    {
	dl_Walk_List( &clp->cl_Items,
		    (dl_Walk_List_t)flush_this_eval,
		    NULL );
	rv=1;
    }
    return(rv);
}


static void
display_list_item( struct config_item *cip, FILE *fh )
{
    fprintf( fh, "Item Key: %s\n", cip->ci_name );
    fprintf( fh, "   Value: %s\n", cip->ci_val );
    if (cip->eval_str)
	fprintf( fh, "    Eval: %s\n", cip->eval_str );
    return;
}

int
dump_private_contents( const char *list_name, FILE *fh )
{
    struct config_list *clp;
    int rv=0;
    clp = get_config_list( list_name, 0 );
    if (clp)
    {
	rv=1;
	fprintf( fh, "List Name: %s\n", clp->cl_name );
	fprintf( fh, "     Time: %s\n", make_timestamp_str( NULL ) );
	dl_Walk_List( &clp->cl_Items,
		(dl_Walk_List_t)display_list_item,
		fh );
    }
    return(rv);
}


/* FIXME - need to create structure with disp_func and param,
	    have a private copy in report_private_contents
	    populate on entry, they retrieve in display_list_item
*/
static void *t_param;


static void
report_private_item( struct config_item *cip,
		    config_disp_t disp_func )
{
    (* disp_func)( cip->ci_name, cip->ci_val, cip->eval_str, t_param );
    return;
}

int
report_private_contents( const char *list_name,
			config_disp_t disp_func,
			void *param )
{
    struct config_list *clp;
    int rv=0;
    clp = get_config_list( list_name, 0 );
    if (clp)
    {
	rv=1;
	t_param = param;
	dl_Walk_List( &clp->cl_Items,
		(dl_Walk_List_t)report_private_item,
		(void *)disp_func );
    }
    return(rv);
}

int
read_private_string_array( const char *list_name, const char *conf_str[] )
{
    int carg = 0;
    int rv=0;

    while ( conf_str[carg] )
	rv += read_private_string( list_name, conf_str[carg++] );
    return(rv);
}


int
read_private_file( const char *list_name, const char *fname )
{
    int rv=0;
    char tbuf[500];
    FILE *fh;

    if ( (fh=fopen(fname,"r")) )
    {
	(void)flush_list_evals( list_name );
	while(fgetline2(fh,tbuf,500)==0)
	{
	    /* might mean something */
	    (void)read_private_string( list_name, tbuf );
	};
	(void)fclose(fh);
	rv=1;
    }
    return(rv);
}

struct config_item *
get_private_config( const char *list_name, const char *key )
{
    struct config_list *clp;
    struct config_item *cip;

    if ( (clp = get_config_list( list_name, 0 )) )
	cip = dl_Find_Item_By_Name( &clp->cl_Items, key );
    else
	cip = NULL;
    return(cip);
}

int
raw_private_string( const char *list_name, const char *key, char *value )
{
    int rv=0;
    struct config_item *cip;

    if ( (cip = get_private_config( list_name, key )) )
    {
	rv = 1;
	strcpy( value, cip->ci_val );
    }
    return(rv);
}


char *
get_private_string( const char *list_name, const char *key )
{
    static char tbuf[MAX_CONFIG_STR];
    char *rv=NULL;
    struct config_item *cip;

    if ( (cip = get_private_config( list_name, key )) )
    {
	strcpy( tbuf, cip->ci_val );
	rv = tbuf;
    }
    return( rv );
}

int
get_private_int( const char *list_name, const char *key )
{
    struct config_item *cip;
    int rv=0;

    if ( (cip = get_private_config( list_name, key )) )
    {
	rv = atoi( cip->ci_val );
    }
    return(rv);
}

double
get_private_double( const char *list_name, const char *key )
{
    struct config_item *cip;
    double rv=0.0;

    if ( (cip = get_private_config( list_name, key )) )
    {
	rv = atof( cip->ci_val );
    }
    return(rv);
}


int
get_private_flag( const char *list_name, const char *key )
{
    struct config_item *cip;
    int rv=0;

    if (   (cip = get_private_config( list_name, key ))
	&& (   strcasecmp(cip->ci_val,"TRUE") == 0
	    || strcasecmp(cip->ci_val,"ON") == 0
	    || strcasecmp(cip->ci_val,"YES") == 0
	    || strcasecmp(cip->ci_val,"AFFIRMATIVE") == 0
	    || strcasecmp(cip->ci_val,"POSITIVE") == 0
	    || strcasecmp(cip->ci_val,"DA") == 0
	    || atoi(cip->ci_val) > 0
	   )
       )
	rv = 1;
    return(rv);
}

int
set_private_int( const char *list_name, const char *key, int newval )
{
    char tbuf[20];
    sprintf( tbuf, "%d", newval );
    return( set_private_string( list_name, key, tbuf ) );
}

int
set_private_double( const char *list_name, const char *key, double newval )
{
    char tbuf[20];
    sprintf( tbuf, "%f", newval );
    return( set_private_string( list_name, key, tbuf ) );
}

int
set_private_flag( const char *list_name, const char *key, int newval )
{
    return( set_private_string( list_name, key,
			(newval)?"TRUE":"FALSE" ) );
}

int
get_list_flags( const char *list_name )
{
    struct config_list *clp;
    int rv;
    if ( (clp = get_config_list( list_name, 0 )) )
	rv = clp->cl_flags;
    else
	rv=0;
    return(rv);
}

int
set_list_flags( const char *list_name, int new_val )
{
    struct config_list *clp;
    int rv;
    if ( (clp = get_config_list( list_name, 0 )) )
    {
	rv = clp->cl_flags;
	clp->cl_flags=new_val;
    }
    else
	rv=0;
    return(rv);
}


char *
eval_private_str( const char *list_name, const char *from )
{
    static char tbuf[500];
    char buf2[500];
    char *rto = tbuf;
    char *ep, *ep2;
    int do_getenv,fallback_env,fallback_config;

    while ( *from )
    {
	if ( *from == '$'  || *from == '%' )
	{
	    do_getenv = ( *from == '$' );
	    /* something to do */
	    from++;
	    fallback_env = fallback_config = 0;
	    if ( *from == '$' && !do_getenv )
	    {
		fallback_env  = 1;
		from++;
	    }else if ( *from == '%' && do_getenv )
	    {
		fallback_config = 1;
		from++;
	    }
	    /* we are now pointing at the environment name,
	       we copy all characters [a-zA-z_-\.:] into the
	       enquire string
	    */
	    ep = buf2;
	    if (*from == '(' || *from == '{')
	    {
		char close;
		close = (*from == '(')?')':'}';
		from++;
		/* read until corresponding close char */
		while ( *from && *from != close )
		    *ep++ = *from++;
		from++;
	    }else
	    {
		while ( *from && 
			( isalpha(*from) || isdigit(*from) ||
			  strchr( "-_.:", *from )
			)
		      )
		    *ep++ = *from++;
	    }
	    *ep = '\0';
	    /* now get the value from the environment */
	    if (do_getenv)
		ep2 = getenv( buf2 );
	    else
		ep2 = get_private_string( list_name, buf2 );
	    if ((!ep2 || !*ep2) && (fallback_env || fallback_config) )
	    {
		if (fallback_env)
		    ep2 = getenv( buf2 );
		else
		    ep2 = get_private_string( list_name, buf2 );
	    }
	    if (ep2)
		while ( *ep2 )
		    *rto++ = *ep2++;
	    
	}else
	    while ( *from && *from != '$' && *from != '%' )
		*rto++ = *from++;
    }
    /* And terminate the buffer */
    *rto = '\0';
    return(tbuf);
}


char *
eval_private_default( const char *list_name,
		      const char *config_name,
		      const char *def_name )
{
    char *cp;
    struct config_item *cip;

    if ( (cip = get_private_config( list_name, config_name )) )
    {
	if (cip->eval_str)
	    cp = cip->eval_str;
	else
	{
	    cp = eval_private_str( list_name, cip->ci_val );
	    if (BITTST(get_list_flags(list_name), CONFIG_NO_EVAL)==0)
	    {
		cip->eval_str = strdup(cp);
		if (cip->eval_str)
		    cp = cip->eval_str;
	    }
	}
    }else if ( def_name )
    {
	cp = eval_private_str( list_name, def_name );
    }else
	cp = NULL;
    return( cp );
}
    
/* -- End of File -- */
