/*
*******************************************************************************
*  Copyright (c) 1994 Martin Poole                            *
*******************************************************************************
**
*******************************************************************************
*/

/* ------------------------------------------------------------------
    Include files
   ------------------------------------------------------------------ */

#include <stdlib.h>
#include <memory.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <signal.h>

#include <mplib1/daemon.h>

/* ------------------------------------------------------------------
    structures / defines
   ------------------------------------------------------------------ */

static int been_signalled=0;
static struct sigaction my_sigaction1;
static struct sigaction my_sigaction2;
static struct sigaction my_sigaction3;
static struct sigaction my_sigaction4;
static struct sigaction my_sigaction5;

static int set_sysint=0;
static struct sigaction save_sigaction1;
static struct sigaction save_sigaction2;
static struct sigaction save_sigaction3;
static struct sigaction save_sigaction4;
static struct sigaction save_sigaction5;

/* ------------------------------------------------------------------
    Code starts here
   ------------------------------------------------------------------ */

static void
myint_handler( int sig )
{
    been_signalled = sig;
    return;
}

static void
set_thisint( int this_sig, struct sigaction *sigactp )
{
    memset( sigactp, '\0', sizeof(struct sigaction) );
    sigactp->sa_flags = SA_RESTART;
    sigactp->sa_handler = myint_handler;
    (void)sigaction( this_sig, sigactp, NULL );
    return;
}

void
init_signalling( void )
{
    set_thisint( SIGINT, &my_sigaction1 );
    set_thisint( SIGTERM, &my_sigaction2 );
    set_thisint( SIGUSR1, &my_sigaction3 );
    set_thisint( SIGUSR2, &my_sigaction4 );
    set_thisint( SIGHUP, &my_sigaction5 );

    signal( SIGCHLD, SIG_IGN );

    return;
}

int
check_signalling( void )
{
    int rv;
    rv = been_signalled;
    been_signalled = 0;
    return(rv);
}

static void
allow_thisint( int this_sig, struct sigaction *sigactp )
{
    struct sigaction prep_sigaction;

    (void)sigaction( this_sig, NULL, sigactp );
    memcpy( &prep_sigaction, sigactp, sizeof(struct sigaction) );
    prep_sigaction.sa_flags &= ~SA_RESTART;
    (void)sigaction( this_sig, &prep_sigaction, NULL );
    return;
}

int
allow_sysint( void )
{
    if (set_sysint==0)
    {
	allow_thisint( SIGINT, &save_sigaction1 );
	allow_thisint( SIGTERM, &save_sigaction2 );
	allow_thisint( SIGUSR1, &save_sigaction3 );
	allow_thisint( SIGUSR2, &save_sigaction4 );
	allow_thisint( SIGHUP, &save_sigaction5 );

	set_sysint=1;
    }
    return(0);
}

int
deny_sysint( void )
{
    if (set_sysint==1)
    {
	(void)sigaction( SIGINT, &save_sigaction1, NULL );
	(void)sigaction( SIGTERM, &save_sigaction2, NULL );
	(void)sigaction( SIGUSR1, &save_sigaction3, NULL );
	(void)sigaction( SIGUSR2, &save_sigaction4, NULL );
	(void)sigaction( SIGHUP, &save_sigaction5, NULL );
	set_sysint=0;
    }
    return(0);
}

/* -- End of File -- */
