/*
*******************************************************************************
*  Copyright (c) 1996 Martin Poole                            *
*******************************************************************************
**
*/

/* ------------------------------------------------------------------
    Include files
   ------------------------------------------------------------------ */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <memory.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/stat.h>
#include <signal.h>
#include <errno.h>

#include <mplib1/cfg_file.h>
#include <mplib1/dl_list.h>
#include <mplib1/bpo.h>
#include <mplib1/fprintfile.h>
#include <mplib1/lock_file.h>
#include <mplib1/daemon.h>

#include "watchdog.h"

/* ------------------------------------------------------------------
    structures / defines
   ------------------------------------------------------------------ */

/* ------------------------------------------------------------------
    Code starts here
   ------------------------------------------------------------------ */

static void usage ( const char *progname, const char *errstr )
{
    if (errstr)
	fprintf( stderr, "Error: %s\n", errstr );
    fprintf(stderr,
	    "\nUsage: %s [ -f filename ] [ -v ]\n", progname );
    exit(1);
    return;
}

int
main ( int argc, char *argv[] )
{
    int optc,verbose =0;
    int cfg_read=0;
    char *cp;
    char config_name[200];
    mode_t fifo_mode;
    void *vp=NULL;
    char *read_file_name=NULL;
    char *lvl_excl=NULL;
    int using_shm=0;

    while ((optc = getopt(argc, argv, "x:f:r:v")) != EOF)
    switch (optc)
    {
    case 'f':
	read_config_file( optarg );
	strcpy( config_name, optarg );
	cfg_read=1;
	break;
    case 'r':
	read_file_name = optarg;
	break;
    case 'v':
	verbose = 1;
	break;
    case 'x': /* Exclude some level immediately */
	lvl_excl = optarg;
	break;
    default:
	usage( argv[0], "Invalid argument supplied" );
	break;
    }
    if (!cfg_read)
    {
	cp = eval_config_default( NULL, "$HOME/config/watchdog.cfg" );
	strcpy( config_name, cp );
/*	fprintf(stderr, "Try to read: %s\n", config_name );*/
	if (!read_config_file( config_name ))
	{
	    usage( argv[0], "Config file not found\n" );
	}
    }
    watch_cfg_file( config_name );

    if (lock_watch_lock()==0)
        return(0);

    /* Now we call the various initialisation routines */
    /* Disconnect */
    if (!get_config_flag("NO_LEAVEHOME"))
	leavehome();
    lock_watch_lock();
    init_signalling();

    /* Set our details */
    fifo_mode = (mode_t)get_config_int( "FIFO_MODE" );
    fifo_mode &= 0666;
    fifo_mode |= 0600;
    Set_My_Proc_Details( NULL, "watchdog", NULL, NULL, fifo_mode, 0 );
    /* Attach to shared memory */
    if (get_config_flag("NO_SHARED_MEMORY")==0)
    {
	using_shm=1;
	vp = comms_shm_attach( eval_config_default( "SHM_CONFIG",
						"$HOME/config/shm.cfg" ),
				SODB_READ_CONFIG );
	if (vp==NULL)
	    usage( argv[0], "Unable to attach to shared memory\n" );
	Register_Process_Details( vp );
	create_watch_queues( vp );
    }

    open_job_file( );
    if (read_file_name)
	read_file_into_db( read_file_name );
    read_job_file( );
    set_all_program_levels();
    if (lvl_excl)
	exclude_named_level( lvl_excl );

/*    dump_jobs( stderr );*/

    watchdog_main_loop( vp );

    close_job_file();

    if (using_shm)
    {
	delete_watch_queues( vp );
	Deregister_Process_Details( vp );
    }

    return(0);
}

/* -- End of File -- */
