/*
*******************************************************************************
*  Copyright (c) 1997 Martin Poole                            *
*******************************************************************************
**
*/

/* ------------------------------------------------------------------
    Include files
   ------------------------------------------------------------------ */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <memory.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <signal.h>

#include <mplib1/fprintfile.h>
#include <mplib1/mpbasename.h>
#include <mplib1/cfg_file.h>
#include <mplib1/lock_file.h>
#include <mplib1/bpo.h>
#include <mplib1/daemon.h>

/* ------------------------------------------------------------------
    structures / defines
   ------------------------------------------------------------------ */

/* ------------------------------------------------------------------
    Code starts here
   ------------------------------------------------------------------ */

static void usage ( const char *progname, const char *errstr )
{
    if (errstr)
	fprintf( stderr, "Error: %s\n", errstr );
    fprintf(stderr,
	    "\nUsage: %s -f filename\n", progname );
    exit(1);
    return;
}

int
main ( int argc, char *argv[] )
{
    int optc,rv;
    int cfg_read=0;
    char *pid_name=NULL;
    void *vp;

    while ((optc = getopt(argc, argv, "f:p:")) != EOF)
    switch (optc)
    {
    case 'f':
	read_config_file( optarg );
	cfg_read=1;
	break;
    case 'p' :
	pid_name = optarg;
	break;
    default:
	usage( argv[0], "Invalid argument supplied" );
	break;
    }
    if (!cfg_read)
    {
	usage( argv[0], "Config file not specified\n" );
    }

    leavehome();

    if (getenv("PROGRAM_NAME")==NULL)
	Set_My_Proc_Details( NULL, mpbasename(argv[0]), NULL, NULL, 0600, 0 );

    if (pid_name)
    {
	if (lock_file(pid_name)==0)
	{
	    fprintfile( stderr, "Unable to lock file %s\n", pid_name );
	    return(1);
	}
    }else
    {
	vp = comms_shm_attach( eval_config_default( "SHM_CONFIG",
						"$HOME/config/shm.cfg" ),
				    SODB_READ_CONFIG );
	if (vp==NULL)
	{
	    fprintfile( stderr, "comms_shm_attach: failed\n" );
	    return(1);
	}
	Register_Process_Details( vp );
    }

    fprintfile( stderr,"Starting\n" );

    rv = get_config_int( "POLL_PERIOD" );
    if (rv<=0)
	rv = 45;
    while(1)
    {
	sleep((unsigned int)rv);
    };
    return(0);
}

/* -- End of File -- */
